<?php

namespace UltimatePostKitPro\Modules\KalonGrid;

use UltimatePostKitPro\Base\Ultimate_Post_Kit_Module_Base;
use Elementor\Icons_Manager;

if (!defined('ABSPATH')) {
	exit;
}
// Exit if accessed directly

class Module extends Ultimate_Post_Kit_Module_Base
{
	public function __construct()
	{
		parent::__construct();

		add_action('wp_ajax_nopriv_upk_kalon_grid_loadmore_posts', [$this, 'callback_ajax_loadmore_posts']);
		add_action('wp_ajax_upk_kalon_grid_loadmore_posts', [$this, 'callback_ajax_loadmore_posts']);
	}
	public function get_name()
	{
		return 'kalon-grid';
	}

	public function get_widgets()
	{

		$widgets = [
			'Kalon_Grid',
		];
		return $widgets;
	}

	private function mapGroupControlQuery($term_ids = [])
	{
		$terms = get_terms(
			[
				'term_taxonomy_id' => $term_ids,
				'hide_empty' => false,
			]
		);

		$tax_terms_map = [];

		foreach ($terms as $term) {
			$taxonomy = $term->taxonomy;
			$tax_terms_map[$taxonomy][] = $term->term_id;
		}

		return $tax_terms_map;
	}

	private function query_args()
	{
		extract($_POST['settings']);

		// setmeta args
		$args = [
			'posts_per_page' => $_POST['per_page'],
			'post_status' => 'publish',
			'suppress_filters' => false,
			'orderby' => $posts_orderby,
			'order' => $posts_order,
			'offset' => $_POST['offset'],
		];
		/**
		 * set feature image
		 *
		 */
		if (isset($posts_only_with_featured_image) && $posts_only_with_featured_image === 'yes') {
			$args['meta_query'] = [
				[
					'key' => '_thumbnail_id',
					'compare' => 'EXISTS',
				],
			];
		}

		/**
		 * set taxonomy query
		 */

		/**
		 * set date query
		 */

		$selected_date = $posts_select_date;

		if (!empty($selected_date)) {
			$date_query = [];

			switch ($selected_date) {
				case 'today':
					$date_query['after'] = '-1 day';
					break;

				case 'week':
					$date_query['after'] = '-1 week';
					break;

				case 'month':
					$date_query['after'] = '-1 month';
					break;

				case 'quarter':
					$date_query['after'] = '-3 month';
					break;

				case 'year':
					$date_query['after'] = '-1 year';
					break;

				case 'exact':
					$after_date = $posts_date_after;
					if (!empty($after_date)) {
						$date_query['after'] = $after_date;
					}

					$before_date = $posts_date_before;
					if (!empty($before_date)) {
						$date_query['before'] = $before_date;
					}
					$date_query['inclusive'] = true;
					break;
			}

			if (!empty($date_query)) {
				$args['date_query'] = $date_query;
			}
		}

		$exclude_by = isset($posts_exclude_by) ? $posts_exclude_by : [];
		$include_by = isset($posts_include_by) ? $posts_include_by : [];
		$include_users = [];
		$exclude_users = [];
		// print_r($exclude_by);
		/**
		 * ignore sticky post
		 */
		if (!empty($exclude_by) && $posts_source === 'post' && $posts_ignore_sticky_posts === 'yes') {
			$args['ignore_sticky_posts'] = true;
			if (in_array('current_post', $exclude_by)) {
				$args['post__not_in'] = [get_the_ID()];
			}
		}

		/**
		 * set post type
		 */

		if ($posts_source === 'manual_selection') {
			/**
			 * Set Including Manually
			 */
			$selected_ids = $posts_selected_ids;
			$selected_ids = wp_parse_id_list($selected_ids);
			$args['post_type'] = 'any';
			if (!empty($selected_ids)) {
				$args['post__in'] = $selected_ids;
			}
			$args['ignore_sticky_posts'] = 1;
		} elseif ('current_query' === $posts_source) {
			/**
			 * Make Current Query
			 */
			$args = $GLOBALS['wp_query']->query_vars;
			$args = apply_filters('element_pack/query/get_query_args/current_query', $args);
		} elseif ('_related_post_type' === $posts_source) {
			/**
			 * Set Related Query
			 */
			$post_id = get_queried_object_id();
			$related_post_id = is_singular() && (0 !== $post_id) ? $post_id : null;
			$args['post_type'] = get_post_type($related_post_id);

			// $include_by = $this->getGroupControlQueryParamBy('include');
			if (in_array('authors', $include_by)) {
				$args['author__in'] = wp_parse_id_list($settings['posts_include_author_ids']);
			} else {
				$args['author__in'] = get_post_field('post_author', $related_post_id);
			}

			// $exclude_by = $this->getGroupControlQueryParamBy('exclude');
			if (in_array('authors', $exclude_by)) {
				$args['author__not_in'] = wp_parse_id_list($posts_exclude_author_ids);
			}

			if (in_array('current_post', $exclude_by)) {
				$args['post__not_in'] = [get_the_ID()];
			}

			$args['ignore_sticky_posts'] = 1;
			$args = apply_filters('element_pack/query/get_query_args/related_query', $args);
		} else {
			$args['post_type'] = $posts_source;
			$current_post = [];

			/**
			 * Set Taxonomy && Set Authors
			 */
			$include_terms = [];
			$exclude_terms = [];
			$terms_query = [];

			/**
			 * Set exclude Terms
			 */
			if (!empty($exclude_by)) {
				if (in_array('authors', $exclude_by)) {
					$exclude_users = wp_parse_id_list($posts_exclude_author_ids);
					$include_users = array_diff($include_users, $exclude_users);
				}
				if (!empty($exclude_users)) {
					$args['author__not_in'] = $exclude_users;
				}
				if (in_array('current_post', $exclude_by) && is_singular()) {
					$current_post[] = get_the_ID();
				}
				if (in_array('manual_selection', $exclude_by)) {
					$exclude_ids = $posts_exclude_ids;
					$args['post__not_in'] = array_merge($current_post, wp_parse_id_list($exclude_ids));
				}
				if (in_array('terms', $exclude_by)) {
					$exclude_terms = wp_parse_id_list($posts_exclude_term_ids);
					$include_terms = array_diff($include_terms, $exclude_terms);
				}
				if (!empty($exclude_terms)) {
					$tax_terms_map = $this->mapGroupControlQuery($exclude_terms);

					foreach ($tax_terms_map as $tax => $terms) {
						$terms_query[] = [
							'taxonomy' => $tax,
							'field' => 'term_id',
							'terms' => $terms,
							'operator' => 'NOT IN',
						];
					}
				}
			}

			/**
			 * Set include Terms
			 */
			if (!empty($include_by)) {

				if (in_array('authors', $include_by)) {
					$include_users = wp_parse_id_list($posts_include_author_ids);
				}
				if (!empty($include_users)) {
					$args['author__in'] = $include_users;
				}
				if (in_array('terms', $include_by)) {
					$include_terms = wp_parse_id_list($posts_include_term_ids);
				}
				$tax_terms_map = $this->mapGroupControlQuery($include_terms);
				foreach ($tax_terms_map as $tax => $terms) {
					$terms_query[] = [
						'taxonomy' => $tax,
						'field' => 'term_id',
						'terms' => $terms,
						'operator' => 'IN',
					];
				}
			}


			if (!empty($terms_query)) {
				$args['tax_query'] = $terms_query;
				$args['tax_query']['relation'] = 'AND';
			}
		}

		$ajaxposts = new \WP_Query($args);
		return $ajaxposts;
	}
	public function callback_ajax_loadmore_posts()
	{
		$settings = $_POST['settings'];
		$post_type =  isset($settings['post_type']) ? sanitize_text_field($$settings['post_type']) : 'post';
		$ajaxposts = $this->query_args();
		$markup = '';
		if ($ajaxposts->have_posts()) {
			$item_index = 1;
			while ($ajaxposts->have_posts()) :
				$ajaxposts->the_post();
				// $title                 = wp_trim_words(get_the_title(), $title_text_limit, '...');
				$title = get_the_title();
				$post_link = esc_url(get_permalink());
				$image_src = wp_get_attachment_image_url(get_post_thumbnail_id(), 'full');
				$category = upk_get_category($post_type);
				$author_url = get_author_posts_url(get_the_author_meta('ID'));
				$author_name = get_the_author();
				$reading_time = ultimate_post_kit_reading_time(get_the_content(), $settings['avg_reading_speed']);

				if ($settings['human_diff_time'] == 'yes') {
					$date =  ultimate_post_kit_post_time_diff(($settings['human_diff_time_short'] == 'yes') ? 'short' : '');
				} else {
					$date =  get_the_date();
				}
				$placeholder_image_src = \Elementor\Utils::get_placeholder_image_src();
				$image_src = wp_get_attachment_image_src(get_post_thumbnail_id(), 'large');
				if (!$image_src) {
					$image_src = $placeholder_image_src;
				} else {
					$image_src = $image_src[0];
				}

				$markup .= '<div class="upk-item">';
				$markup .= '<div class="upk-item-box">';
				$markup .= '<div class="upk-img-wrap">';
				$markup .= '<img alt=' . $post_link . ' class="upk-img" src="' . esc_url($image_src) . '">';
				$markup .= '</div>';
				// end image wrap
				$markup .= '<div class="upk-content-wrap">';
				$markup .= '<div class="upk-content">';
				if ($settings['show_category'] == 'yes') :
					$markup .= '<div class="upk-category">';
					$markup .= $category;
					$markup .= '</div>';
				endif;
				// end category
				if (isset($_POST['settings']['show_title']) && ('yes' == $_POST['settings']['show_title'])) :
					$markup .= '<h3 class="upk-title-wrap">';
					$markup .= '<a class="upk-title" href="' . $post_link . '" title=' . $title . '>' . $title . '</a>';
					$markup .= '</h3>';
				// end title wrap
				endif;

				$markup .= '<div class="upk-meta">';
				$markup .= '<div class="upk-flex-inline">';
				if (isset($_POST['settings']['show_date']) && ('yes' == $_POST['settings']['show_date'])) :
					$markup .= '<div class="upk-date">';
					$markup .= esc_html($date);
					$markup .= '</div>';
				endif;
				// end date
				$markup .= '<div class="upk-reading-time" data-separator="' . $settings['meta_separator'] . '">' . $reading_time . '</div>';
				$markup .= '</div>';
				// end flex inline
				// author start
				if (isset($settings['show_author']) && ('yes' == $settings['show_author'])) :
					$markup .= '<div class="upk-kalon-author">';
					$markup .= '<span class="upk-author-text">by</span><a class="upk-author-name" href="' . $author_url . '">' . esc_html($author_name) . '</a>';
					$markup .= '</div>';
				endif;
				// end author
				$markup .= '</div>';
				// end meta
				$markup .= '</div>';
				// content end

				if ($settings['show_readmore'] == 'yes') :
					$markup .= '<div class="upk-btn-wrap">';
					$markup .= '<a class="upk-btn" href="' . $post_link . '">';
					$markup .= '<span class="upk-flex upk-flex-middle">' . $settings['readmore_text'] . '<span class="upk-btn-icon upk-flex-align-' . $settings['icon_align'] . '">';
					ob_start();

					Icons_Manager::render_icon($settings['readmore_icon'], ['aria-hidden' => 'true', 'class' => 'fa-fw']);
					$markup .= ob_get_clean();

					$markup .= '</div>';
				endif;
				// end button wrap
				$markup .= '</div>';
				// content wrap end
				$markup .= '</div>';
				$markup .= '</div>';
				$item_index++;
			endwhile;
		}

		wp_reset_postdata();
		$result = [
			'markup' => $markup,
		];
		wp_send_json($result);
		exit;
	}
}
